'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _variableStack = require('./variable-stack');

var _variableStack2 = _interopRequireDefault(_variableStack);

var _bl = require('bl');

var _bl2 = _interopRequireDefault(_bl);

var _readableStream = require('readable-stream');

var _lodash = require('lodash');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var LITTLE_ENDIAN = 'LE';
var BIG_ENDIAN = 'BE';
var POW_32 = Math.pow(2, 32);
var LOOP_VAR_SYMBOL = Symbol('loop-variable');

/**
 * This class lies at the very base of this library.
 *
 * It's responsible for reading variables, looping and tapping values and other
 * more or less low-level stuff. It extends the {Transform}-stream, so you can
 * pipe other streams into your corrode instance. This makes corrode an efficient
 * library for even huge files. Unless necessary or explicitly wanted by the user,
 * corrode flushes it's internal buffer {@link Corrode#streamBuffer} asap.
 *
 * The library doesn't work directly when called. If you call e.g.
 * {@link Corrode#uint32be} there won't be a direct read from a given buffer.
 * Instead there will be a new *job*, added to {@link Corrode#jobs}.
 * This array of jobs is responsible for managing the work to be done
 * once data flows. This way, it's possible to insert new jobs on the fly
 * and making for an imperative way of using this library.
 *
 * Corrode will process the given stream as long as long as there's new chunks
 * and jobs. Once either of these drain, corrode will stop and clean possibly
 * remaining jobs (i.e. remove all read-jobs).
 *
 * In fact CorrodeBase can be used as a standalone-library, if low-level is your thing.
 */

var CorrodeBase = function (_Transform) {
    _inherits(CorrodeBase, _Transform);

    _createClass(CorrodeBase, [{
        key: 'vars',


        /**
         * get variables of the current stack
         * @return {Object|*} topmost value
         */


        /**
         * indicates whether automatic flushes are disabled
         * @access public
         * @type {Boolean}
         */


        /**
         * offset in the current running chunk ({@link CorrodeBase#streamBuffer})
         * @access public
         * @type {Number}
         */


        /**
         * internal buffer for stream-chunks not yet processable
         * @access public
         * @type {BufferList}
         */


        /**
         * array holding the jobs which are to be processed
         * @access public
         * @type {Array<Object>}
         */


        /**
         * static constant for little endian
         * @access public
         * @type {string}
         */
        get: function get() {
            return this.varStack.value;
        }

        /**
         * replace the current value
         * @param  {Object|*} val new value
         */


        /**
         * available primitve jobs. these are the functions really reading data from
         * the buffer.
         * @access public
         * @type {Object}
         */


        /**
         * indicates whether only pops are getting processed
         * @access public
         * @type {Boolean}
         */


        /**
         * offset in the whole piped stream
         * @access public
         * @type {Number}
         */


        /**
         * the VariableStack holding the layers and variables of this instance
         * @access public
         * @type {VariableStack}
         */


        /**
         * static constant for big endian
         * @access public
         * @type {string}
         */


        /**
         * Defaults Object for available options
         * {@link CorrodeBase#constructor}
         * @access public
         * @type {Object}
         */
        ,
        set: function set(val) {
            this.varStack.value = val;
        }

        /**
         * options default to {@link Corrode.defaults}, also accepts options
         * for {@link Transform#constructor}.
         * @param {string}  options.endianness=CorrodeBase#LITTLE_ENDIAN endianness, when none is explicitly given by the job
         * @param {*}       options.loopIdentifier=Symbol(loop-variable)    identifier of the temporary variable used internally for loops
         * @param {string}  options.encoding='utf8'                      default encoding, when none is explicitly given by the job
         * @param {boolean} options.finishJobsOnEOF=true                 whether to finish all remaining jobs on stream-end or leave the corrode-instance in an unfinished state with possibly many unresolved functions and unpredictable state (see tests)
         * @param {boolean} options.anonymousLoopDiscardDeep=false       when anonymous loop-jobs get discarded, the original state which gets restored is either a shallow copy or a deep copy of the original object (see tests)
         * @param {boolean} options.strictObjectMode=true                whether to prevent none-object values from being pushed onto {@link CorrodeBase#vars} (catches mistakes)
         */

    }]);

    function CorrodeBase(options) {
        _classCallCheck(this, CorrodeBase);

        /** @type {Object} options default-options overwritten by user options */
        var _this = _possibleConstructorReturn(this, (CorrodeBase.__proto__ || Object.getPrototypeOf(CorrodeBase)).call(this, _extends({}, options, { objectMode: true, encoding: null })));

        _this.jobs = [];
        _this.varStack = new _variableStack2.default();
        _this.streamBuffer = new _bl2.default();
        _this.streamOffset = 0;
        _this.chunkOffset = 0;
        _this.isUnwinding = false;
        _this.isSeeking = false;
        _this.primitveMap = {
            int8: function int8() {
                return _this.streamBuffer.readInt8(_this.chunkOffset);
            },
            uint8: function uint8() {
                return _this.streamBuffer.readUInt8(_this.chunkOffset);
            },

            int16: function int16(job) {
                return _this.streamBuffer['readInt16' + job.endianness](_this.chunkOffset);
            },
            uint16: function uint16(job) {
                return _this.streamBuffer['readUInt16' + job.endianness](_this.chunkOffset);
            },

            int32: function int32(job) {
                return _this.streamBuffer['readInt32' + job.endianness](_this.chunkOffset);
            },
            uint32: function uint32(job) {
                return _this.streamBuffer['readUInt32' + job.endianness](_this.chunkOffset);
            },

            float: function float(job) {
                return _this.streamBuffer['readFloat' + job.endianness](_this.chunkOffset);
            },
            double: function double(job) {
                return _this.streamBuffer['readDouble' + job.endianness](_this.chunkOffset);
            },

            int64: function int64(job) {
                var lo = _this.streamBuffer['readUInt32' + job.endianness](_this.chunkOffset + (job.endianness === LITTLE_ENDIAN ? 0 : 4));
                var hi = _this.streamBuffer['read' + (job.type === 'uint64' ? 'U' : '') + 'Int32' + job.endianness](_this.chunkOffset + (job.endianness === LITTLE_ENDIAN ? 4 : 0));
                return POW_32 * hi + lo;
            },
            uint64: function uint64(job) {
                return _this.primitveMap.int64(job);
            }
        };
        _this.options = _extends({}, CorrodeBase.defaults, options);
        return _this;
    }

    /**
     * add a new chunk to the internal buffer and process remaining jobs
     * this TransformStream won't call the push-method as it is not able
     * to guarantee correct ouptut while still reading data. this depends not on
     * corrode, but on the way you use it.
     * @implements {Transform#_transform}
     * @access private
     */


    _createClass(CorrodeBase, [{
        key: '_transform',
        value: function _transform(chunk, encoding, done) {
            this.streamBuffer.append(chunk);

            this.jobLoop();

            // isSeeking prevents internal flushes
            // (useful for going back to a previous chunk)
            if (!this.isSeeking) {
                this.streamBuffer.consume(this.chunkOffset);
                /**
                 * offset in the current running chunk ({@link CorrodeBase#streamBuffer})
                 * @access public
                 * @type {Number}
                 */
                this.chunkOffset = 0;
            }

            // the callback is called synchronously, meaning any errors
            // working up in the jobLoop (asserts, etc) get thrown in the main
            // event-loop. that's a bug:
            // @link https://github.com/screeny05/corrode/issues/24
            return done();
        }

        /**
         * finish remaining jobs if finishJobsOnEOF is enabled
         * @implements {Transform#_flush}
         * @access private
         */

    }, {
        key: '_flush',
        value: function _flush(done) {
            if (this.options.finishJobsOnEOF) {
                this.finishRemainingJobs();
            }

            setImmediate(done);
        }

        /**
         * internal function to process the array of jobs still remaing.
         * this function is synchronous. this is due to the fact of keeping things
         * more simple. if you intend to use this library expecting taps and loops
         * to work async you're invited to create an issue to inform me of the
         * need for this kind of behaviour. Until then corrode should satisfy most use cases.
         *
         * if you desperately need async behaviour when parsing data, i'd recommend
         * doing that after corrode has finished parsing the buffer, separately.
         *
         * @throws {Error} assertion-errors, runtime-errors
         */

    }, {
        key: 'jobLoop',
        value: function jobLoop() {
            var _this2 = this;

            var _loop = function _loop() {
                var job = _this2.jobs[0];
                var remainingBuffer = _this2.streamBuffer.length - _this2.chunkOffset;

                if (job.type === 'push') {
                    // in strictObjectMode the variable being pushed has to be a real object.
                    // this prevents accidentaly pushing numbers, strings, etc.
                    if (_this2.options.strictObjectMode && typeof _this2.vars[job.name] !== 'undefined' && !(0, _lodash.isPlainObject)(_this2.vars[job.name])) {
                        throw new TypeError('Can\'t push into a non-object value (' + job.name + ') in strictObjectMode');
                    }

                    _this2.jobs.shift();
                    _this2.varStack.push(job.name, job.value);
                    return 'continue';
                } else if (job.type === 'pop') {
                    _this2.jobs.shift();
                    _this2.varStack.pop();
                    return 'continue';
                } else if (job.type === 'tap') {
                    _this2.jobs.shift();

                    var unqueue = _this2.queueJobs();

                    if (typeof job.name !== 'undefined') {
                        // if the tap has a name, push a new var-layer
                        _this2.push(job.name).tap(job.callback, job.args).pop();
                    } else {
                        // otherwise we continue working on the current layer
                        job.callback.apply(_this2, job.args);
                    }

                    unqueue();
                    return 'continue';
                } else if (job.type === 'loop') {
                    // wait for more data before executing a loop on an empty buffer.
                    // this way empty objects are not being added when the stream finishes
                    if (remainingBuffer === 0) {
                        return 'break';
                    }

                    if (job.finished) {
                        _this2.jobs.shift();
                        return 'continue';
                    }

                    var loopIdentifier = _this2.options.loopIdentifier;
                    var _unqueue = _this2.queueJobs();

                    if (typeof job.name !== 'undefined') {
                        if (typeof _this2.vars[job.name] === 'undefined') {
                            _this2.vars[job.name] = [];
                        }

                        _this2.tap(loopIdentifier, job.callback, [job.finish, job.discard, job.iteration++]).tap(function () {
                            var loopResult = this.vars[loopIdentifier];

                            // push vars only if job isn't discarded and yielded vars
                            // (no empty objects this way)
                            if (!job.discarded && (!(0, _lodash.isPlainObject)(loopResult) || Object.keys(loopResult).length > 0)) {
                                this.vars[job.name].push(loopResult);
                            }
                            job.discarded = false;
                            delete this.vars[loopIdentifier];
                        });
                    } else {
                        // make copy, in case the user discards the result
                        // {@link CorrodeBase#options.anonymousLoopDiscardDeep}
                        if (_this2.options.anonymousLoopDiscardDeep) {
                            job[loopIdentifier] = (0, _lodash.cloneDeep)(_this2.vars);
                        } else {
                            job[loopIdentifier] = _extends({}, _this2.vars);
                        }

                        _this2.tap(job.callback, [job.finish, job.discard, job.iteration++]).tap(function () {
                            if (job.discarded && typeof job[loopIdentifier] !== 'undefined') {
                                this.vars = job[loopIdentifier];
                            }
                            job.discarded = false;
                            delete job[loopIdentifier];
                        });
                    }

                    _unqueue();
                    return 'continue';
                }

                // determine length of next job
                var length = typeof job.length === 'string' ? _this2.vars[job.length] : job.length;

                // only valid numbers can be used as length
                if (typeof length !== 'number') {
                    throw new TypeError('Cannot find a valid length for job ' + job.name + ', dereferenced length is ' + JSON.stringify(length));
                }

                // break on unsufficient streamBuffer-length (wait if not unwinding yet)
                if (_this2.streamBuffer.length - _this2.chunkOffset < length) {
                    if (_this2.isUnwinding && _this2.jobs.length > 0) {
                        // unwind loop, by removing the loop job
                        _this2.removeReadJobs();
                        return 'continue';
                    }

                    return 'break';
                }

                if (job.type === 'buffer') {
                    _this2.jobs.shift();
                    _this2.vars[job.name] = _this2.streamBuffer.slice(_this2.chunkOffset, _this2.chunkOffset + length);
                    _this2._moveOffset(length);
                    return 'continue';
                } else if (job.type === 'string') {
                    _this2.jobs.shift();
                    _this2.vars[job.name] = _this2.streamBuffer.toString(job.encoding, _this2.chunkOffset, _this2.chunkOffset + length);
                    _this2._moveOffset(length);
                    return 'continue';
                } else if (job.type === 'skip') {
                    _this2.jobs.shift();
                    if (_this2.streamOffset + length < 0) {
                        throw new RangeError('cannot skip below 0');
                    }
                    _this2._moveOffset(length);
                    return 'continue';
                } else if (typeof _this2.primitveMap[job.type] === 'function') {
                    _this2.vars[job.name] = _this2.primitveMap[job.type](job);
                    _this2.jobs.shift();
                    _this2._moveOffset(length);
                } else {
                    throw new Error('invalid job type \'' + job.type + '\'');
                }
            };

            // {@link CorrodeBase#jobs} get's manipulated by {@link CorrodeBase#jobLoop}
            _loop2: while (this.jobs.length > 0) {
                var _ret = _loop();

                switch (_ret) {
                    case 'continue':
                        continue;

                    case 'break':
                        break _loop2;}
            }
        }

        /**
         * re-starts the jobLoop with the job-list cleaned from any read jobs
         * {@link CorrodeBase#removeReadJobs}
         * {@link CorrodeBase#isUnwinding}
         */

    }, {
        key: 'finishRemainingJobs',
        value: function finishRemainingJobs() {
            /**
             * indicates whether only pops are getting processed
             * @access public
             * @type {Boolean}
             */
            this.isUnwinding = true;
            this.removeReadJobs();
            this.jobLoop();
            this.varStack.popAll();
        }

        /**
         * purges all jobs from the job-queue, which need to read from the stream
         */

    }, {
        key: 'removeReadJobs',
        value: function removeReadJobs() {
            var _jobs;

            var filteredJobs = this.jobs.slice().filter(function (job) {
                return job.type === 'pop' || job.type === 'tap' && !job.name;
            });

            this.jobs.splice(0);
            (_jobs = this.jobs).push.apply(_jobs, _toConsumableArray(filteredJobs));
        }

        /**
         * utility method to move both {@link CorrodeBase#chunkOffset} and
         * {@link CorrodeBase#streamOffset} by a given amount of bytes
         * @access private
         */

    }, {
        key: '_moveOffset',
        value: function _moveOffset(by) {
            this.chunkOffset += by;
            /**
             * offset in the whole piped stream
             * @access public
             * @type {Number}
             */
            this.streamOffset += by;
        }

        /**
         * remove all jobs from {@link CorrodeBase#jobs} with the ability to re-add them back later.
         * @returns {function} function to append all queued jobs back onto the jobs-array (borrowed from redux)
         * @example
         *   + [
         *   |   { type: 'push', name: 'struct', fn },
         *   |   { type: 'uint8', name: 'var' ' }
         *   | ]
         *   |
         *   | jobLoop - iteration
         *   |
         *   | var unqueueJobs = queJobs();
         *   |
         *   | []
         *   |
         *   | fn();
         *   |
         *   | [
         *   |   { type: 'uint8', name: 'strvar' }
         *   | ]
         *   |
         *   | unqueueJobs();
         *   |
         *   | [
         *   |   { type: 'uint8', name: 'strvar' },
         *   |   { type: 'uint8', name: 'var'   }
         *   v ]
         */

    }, {
        key: 'queueJobs',
        value: function queueJobs() {
            var _this3 = this;

            var queuedJobs = this.jobs;

            // empty jobs
            this.jobs = [];

            // unqueue-method
            return function () {
                return _this3.jobs = _this3.jobs.concat(queuedJobs);
            };
        }

        /**
         * utility method to push a new job onto the job-array
         * @access private
         * @return {CorrodeBase} this
         */

    }, {
        key: '_pushJob',
        value: function _pushJob(name, type, length, endianness) {
            this.jobs.push((typeof name === 'undefined' ? 'undefined' : _typeof(name)) === 'object' ? name : {
                name: name,
                type: type,
                length: length,
                endianness: endianness
            });
            return this;
        }

        /**
         * push a tap-job onto the job-array
         * a tap-job is a special kind of job allowing the developer to peek into
         * the current variables and creating more complex structures and behaviour
         * based on available information.
         * @param {string} [name] name of the new variable-layer. if none is provided, the current layer will be used
         * @param {function(...args: *)} callback called when this job is reached
         * @param {Array} args array of possible arguments being passed to the callback
         * @return {CorrodeBase} this
         */

    }, {
        key: 'tap',
        value: function tap(name, callback, args) {
            if (typeof name === 'function') {
                args = callback;
                callback = name;
                name = undefined;
            }

            return this._pushJob({
                name: name,
                type: 'tap',
                args: args,
                callback: callback
            });
        }

        /**
         * push a loop-job onto the job-array
         * a loop-job is a special kind of job allowing the developer to create loops
         * in the current job-array. this allows for iteration, seeking and more
         * complex behaviours
         * @param {string} [name] name of the new variable-layer. if none is provided, the current layer will be used
         * @param {function(end: function, discard: function, i: number)} callback called until `end()` is called.
         *        `end(true)` can be used to end and discard the current loop.
         *        `discard()` can be used to reset the current layer ({@link CorrodeBase#options.anonymousLoopDiscardDeep}).
         *        `i` is the current iteration count
         * @return {CorrodeBase} this
         */

    }, {
        key: 'loop',
        value: function loop(name, callback) {
            if (typeof name === 'function') {
                callback = name;
                name = undefined;
            }

            var loopJob = {
                name: name,
                type: 'loop',
                callback: callback,
                finished: false,
                discarded: false,
                iteration: 0
            };

            loopJob.finish = function (discard) {
                loopJob.finished = true;
                loopJob.discarded = Boolean(discard);
            };

            loopJob.discard = function () {
                loopJob.discarded = true;
            };

            return this._pushJob(loopJob);
        }

        /**
         * push a skip-job onto the job-array
         * skip-jobs allow the developer to skip a given number of bytes.
         * If the amount of bytes exceeds the current available byte-count in the
         * internal buffer {@link CorrodeBase#streamBuffer} the job will wait for enough
         * data. If this data won't come the job gets aborted and corrode ends.
         * If you want to skip a negative amount of bytes you have to disable auto-flushing.
         * this can be done by setting {@link CorrodeBase#isSeeking} to `true`.
         * @param {number|string} length how many bytes to skip. Given a string, corrode will try to find a variable with the given string in the current layer
         * @return {CorrodeBase} this
         */

    }, {
        key: 'skip',
        value: function skip(length) {
            return this._pushJob({
                type: 'skip',
                length: length
            });
        }

        /**
         * push a pop-job onto the job-array
         * the pop-job pops a layer from {@link CorrodeBase#varStack}.
         * this most probably doesn't have to get called manually, as
         * {@link CorrodeBase#tap} and {@link CorrodeBase#loop} will do this automatically
         * @return {CorrodeBase} this
         */

    }, {
        key: 'pop',
        value: function pop() {
            return this._pushJob({
                type: 'pop'
            });
        }

        /**
         * push a push-job onto the job-array
         * the push-job pushes a new layer onto {@link CorrodeBase#varStack}.
         * this most probably doesn't have to get called manually, as
         * {@link CorrodeBase#tap} and {@link CorrodeBase#loop} will do this automatically
         * @param {string} name name of the new layer
         * @param {*} [value={}] value of the new layer (default is from {@link VariableStack})
         * @return {CorrodeBase} this
         */

    }, {
        key: 'push',
        value: function push(name, value) {
            return this._pushJob({
                type: 'push',
                name: name,
                value: value
            });
        }

        /**
         * pushes a buffer-job onto the job-array
         * the buffer-job will read a buffer with the given length starting at the
         * current offset {@link CorrodeBase#bufferOffset}.
         * The returned Buffer will be a slice (not a copy) of the underlying {@link CorrodeBase#streamBuffer}
         * @param {string} name name of the buffer-variable
         * @param {number|string} length length of the buffer in bytes. Given a string, corrode will try to find a variable with the given string in the current layer
         * @return {CorrodeBase} this
         */

    }, {
        key: 'buffer',
        value: function buffer(name, length) {
            return this._pushJob({
                name: name,
                type: 'buffer',
                length: length
            });
        }

        /**
         * pushes a string-job onto the job-array
         * the string-job will read a string with the given length starting at the
         * current offset {@link CorrodeBase#bufferOffset}
         * @param {string} name name of the string-variable
         * @param {number|string} length length of the string in bytes (_not characters_). Given a string, corrode will try to find a variable with the given string in the current layer
         * @param {string} [encoding=CorrodeBase#options.encoding] encoding encoding used to decode the string, defaults to 'utf8'.
         *                 available encodings can be found here https://nodejs.org/api/buffer.html#buffer_buffers_and_character_encodings
         * @return {CorrodeBase} this
         */

    }, {
        key: 'string',
        value: function string(name, length) {
            var encoding = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : this.options.encoding;

            return this._pushJob({
                name: name,
                type: 'string',
                length: length,
                encoding: encoding
            });
        }

        /**
         * push a int8 (signed) job onto the job-array
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'int8',
        value: function int8(name) {
            return this._pushJob(name, 'int8', 1, this.options.endianness);
        }

        /**
         * push a int8 (signed) job onto the job-array
         * @deprecated int8 needs no endianness, use int8() instead
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'int8le',
        value: function int8le(name) {
            return this._pushJob(name, 'int8', 1, LITTLE_ENDIAN);
        }

        /**
         * push a int8 (signed) job onto the job-array
         * @deprecated int8 needs no endianness, use int8() instead
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'int8be',
        value: function int8be(name) {
            return this._pushJob(name, 'int8', 1, BIG_ENDIAN);
        }

        /**
         * push a uint8 (unsigned) job onto the job-array
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'uint8',
        value: function uint8(name) {
            return this._pushJob(name, 'uint8', 1, this.options.endianness);
        }

        /**
         * push a uint8 (unsigned) job onto the job-array
         * @deprecated uint8 needs no endianness, use uint8() instead
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'uint8le',
        value: function uint8le(name) {
            return this._pushJob(name, 'uint8', 1, LITTLE_ENDIAN);
        }

        /**
         * push a uint8 (unsigned) job onto the job-array
         * @deprecated uint8 needs no endianness, use uint8() instead
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'uint8be',
        value: function uint8be(name) {
            return this._pushJob(name, 'uint8', 1, BIG_ENDIAN);
        }

        /**
         * push a int16 (signed) job with default endianness ({@link CorrodeBase#options.endianness}) onto the job-array
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'int16',
        value: function int16(name) {
            return this._pushJob(name, 'int16', 2, this.options.endianness);
        }

        /**
         * push a int16 (signed) job with little endianness onto the job-array
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'int16le',
        value: function int16le(name) {
            return this._pushJob(name, 'int16', 2, LITTLE_ENDIAN);
        }

        /**
         * push a int16 (signed) job with big endianness onto the job-array
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'int16be',
        value: function int16be(name) {
            return this._pushJob(name, 'int16', 2, BIG_ENDIAN);
        }

        /**
         * push a uint16 (unsigned) job with default endianness ({@link CorrodeBase#options.endianness}) onto the job-array
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'uint16',
        value: function uint16(name) {
            return this._pushJob(name, 'uint16', 2, this.options.endianness);
        }

        /**
         * push a uint16 (unsigned) job with little endianness onto the job-array
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'uint16le',
        value: function uint16le(name) {
            return this._pushJob(name, 'uint16', 2, LITTLE_ENDIAN);
        }

        /**
         * push a uint16 (unsigned) job with big endianness onto the job-array
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'uint16be',
        value: function uint16be(name) {
            return this._pushJob(name, 'uint16', 2, BIG_ENDIAN);
        }

        /**
         * push a int32 (signed) job with default endianness ({@link CorrodeBase#options.endianness}) onto the job-array
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'int32',
        value: function int32(name) {
            return this._pushJob(name, 'int32', 4, this.options.endianness);
        }

        /**
         * push a int32 (signed) job with little endianness onto the job-array
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'int32le',
        value: function int32le(name) {
            return this._pushJob(name, 'int32', 4, LITTLE_ENDIAN);
        }

        /**
         * push a int32 (signed) job with big endianness onto the job-array
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'int32be',
        value: function int32be(name) {
            return this._pushJob(name, 'int32', 4, BIG_ENDIAN);
        }

        /**
         * push a uint32 (unsigned) job with default endianness ({@link CorrodeBase#options.endianness}) onto the job-array
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'uint32',
        value: function uint32(name) {
            return this._pushJob(name, 'uint32', 4, this.options.endianness);
        }

        /**
         * push a uint32 (unsigned) job with little endianness onto the job-array
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'uint32le',
        value: function uint32le(name) {
            return this._pushJob(name, 'uint32', 4, LITTLE_ENDIAN);
        }

        /**
         * push a uint32 (unsigned) job with big endianness onto the job-array
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'uint32be',
        value: function uint32be(name) {
            return this._pushJob(name, 'uint32', 4, BIG_ENDIAN);
        }

        /**
         * push a int64 (signed) job with default endianness ({@link CorrodeBase#options.endianness}) onto the job-array
         * note that in64 may be unprecise, due to number-values being double in js
         * @link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Data_structures#Number_type
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'int64',
        value: function int64(name) {
            return this._pushJob(name, 'int64', 8, this.options.endianness);
        }

        /**
         * push a int64 (signed) job with little endianness onto the job-array
         * note that in64 may be unprecise, due to number-values being double in js
         * @link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Data_structures#Number_type
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'int64le',
        value: function int64le(name) {
            return this._pushJob(name, 'int64', 8, LITTLE_ENDIAN);
        }

        /**
         * push a int64 (signed) job with big endianness onto the job-array
         * note that in64 may be unprecise, due to number-values being double in js
         * @link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Data_structures#Number_type
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'int64be',
        value: function int64be(name) {
            return this._pushJob(name, 'int64', 8, BIG_ENDIAN);
        }

        /**
         * push a uint64 (unsigned) job with default endianness ({@link CorrodeBase#options.endianness}) onto the job-array
         * note that in64 may be unprecise, due to number-values being double in js
         * @link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Data_structures#Number_type
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'uint64',
        value: function uint64(name) {
            return this._pushJob(name, 'uint64', 8, this.options.endianness);
        }

        /**
         * push a uint64 (unsigned) job with little endianness onto the job-array
         * note that in64 may be unprecise, due to number-values being double in js
         * @link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Data_structures#Number_type
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'uint64le',
        value: function uint64le(name) {
            return this._pushJob(name, 'uint64', 8, LITTLE_ENDIAN);
        }

        /**
         * push a uint64 (unsigned) job with big endianness onto the job-array
         * note that in64 may be unprecise, due to number-values being double in js
         * @link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Data_structures#Number_type
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'uint64be',
        value: function uint64be(name) {
            return this._pushJob(name, 'uint64', 8, BIG_ENDIAN);
        }

        /**
         * push a float job with default endianness ({@link CorrodeBase#options.endianness}) onto the job-array
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'float',
        value: function float(name) {
            return this._pushJob(name, 'float', 4, this.options.endianness);
        }

        /**
         * push a float job with little endianness onto the job-array
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'floatle',
        value: function floatle(name) {
            return this._pushJob(name, 'float', 4, LITTLE_ENDIAN);
        }

        /**
         * push a float job with big endianness onto the job-array
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'floatbe',
        value: function floatbe(name) {
            return this._pushJob(name, 'float', 4, BIG_ENDIAN);
        }

        /**
         * push a double job with default endianness ({@link CorrodeBase#options.endianness}) onto the job-array
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'double',
        value: function double(name) {
            return this._pushJob(name, 'double', 8, this.options.endianness);
        }

        /**
         * push a double job with little endianness onto the job-array
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'doublele',
        value: function doublele(name) {
            return this._pushJob(name, 'double', 8, LITTLE_ENDIAN);
        }

        /**
         * push a double job with big endianness onto the job-array
         * @param {string} name name of the variable to be created
         * @return {CorrodeBase} this
         */

    }, {
        key: 'doublebe',
        value: function doublebe(name) {
            return this._pushJob(name, 'double', 8, BIG_ENDIAN);
        }
    }]);

    return CorrodeBase;
}(_readableStream.Transform);

CorrodeBase.defaults = {
    endianness: LITTLE_ENDIAN,
    loopIdentifier: LOOP_VAR_SYMBOL,
    encoding: 'utf8',
    finishJobsOnEOF: true,
    anonymousLoopDiscardDeep: false,
    strictObjectMode: true
};
CorrodeBase.LITTLE_ENDIAN = LITTLE_ENDIAN;
CorrodeBase.BIG_ENDIAN = BIG_ENDIAN;
exports.default = CorrodeBase;
module.exports = exports['default'];
//# sourceMappingURL=base.js.map