'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.tapBindObject = tapBindObject;
exports.bindObject = bindObject;

var _lodash = require('lodash');

/**
 * bind each function in an object with a tap to a given context
 * @param  {object} obj object with functions
 * @param  {object} ctx context
 * @return {object}     copy of object with each function wrapped in a tap
 */
function tapBindObject(obj, ctx) {
    return (0, _lodash.mapValues)(obj, function (fn) {
        return typeof fn === 'function' ? function () {
            for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
                args[_key] = arguments[_key];
            }

            return ctx.tap(fn.bind.apply(fn, [ctx].concat(args)));
        } : fn;
    });
}

/**
 * bind each function in an object to a given context
 * @param  {object} obj object with functions
 * @param  {object} ctx context
 * @return {object}     copy of object obj with each function bound to ctx
 */
function bindObject(obj, ctx) {
    return (0, _lodash.mapValues)(obj, function (fn) {
        return typeof fn === 'function' ? fn.bind(ctx) : fn;
    });
}
//# sourceMappingURL=index.js.map